summary refs log tree commit diff
path: root/crypto/src/asn1/DerIA5String.cs
diff options
context:
space:
mode:
Diffstat (limited to 'crypto/src/asn1/DerIA5String.cs')
-rw-r--r--crypto/src/asn1/DerIA5String.cs135
1 files changed, 74 insertions, 61 deletions
diff --git a/crypto/src/asn1/DerIA5String.cs b/crypto/src/asn1/DerIA5String.cs
index d608930fd..a56879831 100644
--- a/crypto/src/asn1/DerIA5String.cs
+++ b/crypto/src/asn1/DerIA5String.cs
@@ -1,30 +1,56 @@
 using System;
-using System.Text;
+using System.IO;
 
 using Org.BouncyCastle.Utilities;
 
 namespace Org.BouncyCastle.Asn1
 {
     /**
-     * Der IA5String object - this is an ascii string.
+     * IA5String object - this is an Ascii string.
      */
     public class DerIA5String
         : DerStringBase
     {
-        private readonly string str;
+        internal class Meta : Asn1UniversalType
+        {
+            internal static readonly Asn1UniversalType Instance = new Meta();
+
+            private Meta() : base(typeof(DerIA5String), Asn1Tags.IA5String) {}
+
+            internal override Asn1Object FromImplicitPrimitive(DerOctetString octetString)
+            {
+                return CreatePrimitive(octetString.GetOctets());
+            }
+        }
 
         /**
-         * return a IA5 string from the passed in object
+         * return an IA5 string from the passed in object
          *
          * @exception ArgumentException if the object cannot be converted.
          */
-        public static DerIA5String GetInstance(
-            object obj)
+        public static DerIA5String GetInstance(object obj)
         {
             if (obj == null || obj is DerIA5String)
             {
                 return (DerIA5String)obj;
             }
+            else if (obj is IAsn1Convertible)
+            {
+                Asn1Object asn1Object = ((IAsn1Convertible)obj).ToAsn1Object();
+                if (asn1Object is DerIA5String)
+                    return (DerIA5String)asn1Object;
+            }
+            else if (obj is byte[])
+            {
+                try
+                {
+                    return (DerIA5String)Meta.Instance.FromByteArray((byte[])obj);
+                }
+                catch (IOException e)
+                {
+                    throw new ArgumentException("failed to construct IA5 string from byte[]: " + e.Message);
+                }
+            }
 
             throw new ArgumentException("illegal object in GetInstance: " + Platform.GetTypeName(obj));
         }
@@ -32,40 +58,18 @@ namespace Org.BouncyCastle.Asn1
         /**
          * return an IA5 string from a tagged object.
          *
          * @param obj the tagged object holding the object we want
          * @param explicitly true if the object is meant to be explicitly
          *              tagged false otherwise.
          * @exception ArgumentException if the tagged object cannot
          *               be converted.
          */
-        public static DerIA5String GetInstance(
-            Asn1TaggedObject	obj,
-            bool				isExplicit)
+        public static DerIA5String GetInstance(Asn1TaggedObject taggedObject, bool declaredExplicit)
         {
-			Asn1Object o = obj.GetObject();
-
-			if (isExplicit || o is DerIA5String)
-			{
-				return GetInstance(o);
-			}
-
-			return new DerIA5String(((Asn1OctetString)o).GetOctets());
+            return (DerIA5String)Meta.Instance.GetContextInstance(taggedObject, declaredExplicit);
         }
 
-        /**
-         * basic constructor - with bytes.
-         */
-        public DerIA5String(
-            byte[] str)
-            : this(Strings.FromAsciiByteArray(str), false)
-        {
-        }
+        private readonly byte[] m_contents;
 
-		/**
-		* basic constructor - without validation.
-		*/
-		public DerIA5String(
-			string str)
+		public DerIA5String(string str)
 			: this(str, false)
 		{
 		}
@@ -78,72 +82,81 @@ namespace Org.BouncyCastle.Asn1
 		* @throws ArgumentException if validate is true and the string
 		* contains characters that should not be in an IA5String.
 		*/
-		public DerIA5String(
-			string	str,
-			bool	validate)
+		public DerIA5String(string str, bool validate)
 		{
 			if (str == null)
 				throw new ArgumentNullException("str");
 			if (validate && !IsIA5String(str))
 				throw new ArgumentException("string contains illegal characters", "str");
 
-			this.str = str;
+			m_contents = Strings.ToAsciiByteArray(str);
 		}
 
-		public override string GetString()
+        public DerIA5String(byte[] contents)
+            : this(contents, true)
+        {
+        }
+
+        internal DerIA5String(byte[] contents, bool clone)
+        {
+            if (null == contents)
+                throw new ArgumentNullException("contents");
+
+            m_contents = clone ? Arrays.Clone(contents) : contents;
+        }
+
+        public override string GetString()
         {
-            return str;
+            return Strings.FromAsciiByteArray(m_contents);
         }
 
 		public byte[] GetOctets()
         {
-            return Strings.ToAsciiByteArray(str);
+            return Arrays.Clone(m_contents);
         }
 
         internal override IAsn1Encoding GetEncoding(int encoding)
         {
-            return new PrimitiveEncoding(Asn1Tags.Universal, Asn1Tags.IA5String, GetOctets());
+            return new PrimitiveEncoding(Asn1Tags.Universal, Asn1Tags.IA5String, m_contents);
         }
 
         internal override IAsn1Encoding GetEncodingImplicit(int encoding, int tagClass, int tagNo)
         {
-            return new PrimitiveEncoding(tagClass, tagNo, GetOctets());
+            return new PrimitiveEncoding(tagClass, tagNo, m_contents);
         }
 
-        protected override int Asn1GetHashCode()
-		{
-            return this.str.GetHashCode();
+        protected override bool Asn1Equals(Asn1Object asn1Object)
+        {
+            DerIA5String that = asn1Object as DerIA5String;
+            return null != that
+                && Arrays.AreEqual(this.m_contents, that.m_contents);
         }
 
-		protected override bool Asn1Equals(
-            Asn1Object asn1Object)
+        protected override int Asn1GetHashCode()
         {
-			DerIA5String other = asn1Object as DerIA5String;
-
-			if (other == null)
-				return false;
-
-			return this.str.Equals(other.str);
+            return Arrays.GetHashCode(m_contents);
         }
 
-		/**
+        /**
 		 * return true if the passed in String can be represented without
 		 * loss as an IA5String, false otherwise.
 		 *
 		 * @return true if in printable set, false otherwise.
 		 */
-		public static bool IsIA5String(
-			string str)
+        public static bool IsIA5String(string str)
 		{
 			foreach (char ch in str)
 			{
 				if (ch > 0x007f)
-				{
 					return false;
-				}
 			}
 
 			return true;
 		}
-	}
+
+        internal static DerIA5String CreatePrimitive(byte[] contents)
+        {
+            return new DerIA5String(contents, false);
+        }
+    }
 }
@@ -32,40 +58,18 @@ namespace Org.BouncyCastle.Asn1
          * return an IA5 string from a tagged object.
          *
-         * @param obj the tagged object holding the object we want
-         * @param explicitly true if the object is meant to be explicitly
-         *              tagged false otherwise.
-         * @exception ArgumentException if the tagged object cannot
-         *               be converted.
+         * @param taggedObject the tagged object holding the object we want
+         * @param declaredExplicit true if the object is meant to be explicitly tagged false otherwise.
+         * @exception ArgumentException if the tagged object cannot be converted.
          */